@@ -19,6 +19,7 @@
 package org.eclipse.core.internal.events;
 
 import java.util.*;
+import java.util.function.Supplier;
 import java.util.stream.Collectors;
 import org.eclipse.core.internal.dtree.DeltaDataTree;
 import org.eclipse.core.internal.resources.*;
@@ -34,42 +35,39 @@ import org.osgi.framework.Bundle;
 
 public class BuildManager implements ICoreConstants, IManager, ILifecycleListener {
 
+	private static final String BUILDER_INIT = "BuilderInitInfo"; //$NON-NLS-1$
+
 	/**
 	 * Cache used to optimize the common case of an autobuild against
 	 * a workspace where only a single project has changed (and hence
 	 * only a single delta is interesting).
 	 */
-	class DeltaCache {
-		private Object delta;
+	static class DeltaCache<E> {
+		private final Map<IPath, E> deltas = new HashMap<>();
 		private ElementTree newTree;
 		private ElementTree oldTree;
-		private IPath projectPath;
-
-		public void cache(IPath project, ElementTree anOldTree, ElementTree aNewTree, Object aDelta) {
-			this.projectPath = project;
-			this.oldTree = anOldTree;
-			this.newTree = aNewTree;
-			this.delta = aDelta;
-		}
 
 		public void flush() {
-			this.projectPath = null;
+			deltas.clear();
 			this.oldTree = null;
 			this.newTree = null;
-			this.delta = null;
 		}
 
 		/**
 		 * Returns the cached resource delta for the given project and trees, or
-		 * null if there is no matching delta in the cache.
+		 * calls calculator to compute a new delta if there is no matching one in the cache.
 		 */
-		public Object getDelta(IPath project, ElementTree anOldTree, ElementTree aNewTree) {
-			if (delta == null)
-				return null;
-			boolean pathsEqual = projectPath == null ? project == null : projectPath.equals(project);
-			if (pathsEqual && this.oldTree == anOldTree && this.newTree == aNewTree)
-				return delta;
-			return null;
+		public E computeIfAbsent(IPath project, ElementTree anOldTree, ElementTree aNewTree, Supplier<E> calculator) {
+			if (!(areEqual(this.oldTree, anOldTree) && areEqual(this.newTree, aNewTree))) {
+				this.oldTree = anOldTree;
+				this.newTree = aNewTree;
+				deltas.clear();
+			}
+			return deltas.computeIfAbsent(project, p -> calculator.get());
+		}
+
+		private static boolean areEqual(ElementTree cached, ElementTree requested) {
+			return !ElementTree.hasChanges(requested, cached, ResourceComparator.getBuildComparator(), true);
 		}
 	}
 
@@ -123,11 +121,11 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 	/**
 	 * Caches the IResourceDelta for a pair of trees
 	 */
-	final private DeltaCache deltaCache = new DeltaCache();
+	final private DeltaCache<IResourceDelta> deltaCache = new DeltaCache<>();
 	/**
 	 * Caches the DeltaDataTree used to determine if a build is necessary
 	 */
-	final private DeltaCache deltaTreeCache = new DeltaCache();
+	final private DeltaCache<DeltaDataTree> deltaTreeCache = new DeltaCache<>();
 
 	private ILock lock;
 
@@ -136,6 +134,9 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 
 	private final Bundle systemBundle = Platform.getBundle("org.eclipse.osgi"); //$NON-NLS-1$
 
+	// protects against concurrent access of session stored builders during builder initialization
+	private Object builderInitializationLock = new Object();
+
 	//used for debug/trace timing
 	private long timeStamp = -1;
 	private long overallTimeStamp = -1;
@@ -394,9 +395,7 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 				hookEndBuild(trigger);
 			}
 		} finally {
-			monitor.done();
-			if (trigger == IncrementalProjectBuilder.INCREMENTAL_BUILD || trigger == IncrementalProjectBuilder.FULL_BUILD)
-				autoBuildJob.avoidBuild();
+			endBuild(trigger, monitor);
 		}
 	}
 
@@ -420,9 +419,18 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 				hookEndBuild(trigger);
 			}
 		} finally {
-			monitor.done();
-			if (trigger == IncrementalProjectBuilder.INCREMENTAL_BUILD || trigger == IncrementalProjectBuilder.FULL_BUILD)
-				autoBuildJob.avoidBuild();
+			endBuild(trigger, monitor);
+		}
+	}
+
+	private void endBuild(int trigger, IProgressMonitor monitor) {
+		boolean cancelledBuild = monitor.isCanceled();
+		monitor.done();
+		if (trigger == IncrementalProjectBuilder.INCREMENTAL_BUILD || trigger == IncrementalProjectBuilder.FULL_BUILD) {
+			autoBuildJob.avoidBuild();
+		} else if (cancelledBuild) {
+			// Bug 538789: if a build was explicitly cancelled, don't trigger auto-build jobs until a build is requested
+			autoBuildJob.avoidBuildIfNotInterrupted();
 		}
 	}
 
@@ -597,24 +605,51 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 	private IncrementalProjectBuilder getBuilder(IBuildConfiguration buildConfiguration, ICommand command, int buildSpecIndex, MultiStatus status) throws CoreException {
 		BuildCommand buildCommand = (BuildCommand) command;
 		InternalBuilder result = buildCommand.getBuilder(buildConfiguration);
+		String builderName = command.getBuilderName();
+		IProject project = buildConfiguration.getProject();
+
 		if (result == null) {
-			result = initializeBuilder(command.getBuilderName(), buildConfiguration, buildSpecIndex, status);
-			result.setCommand(command);
-			result.setBuildConfig(buildConfiguration);
-			result.startupOnInitialize();
-			buildCommand.addBuilder(buildConfiguration, (IncrementalProjectBuilder) result);
-			// the build command holds only one builder per configuration
-			// so query the builder for the configuration once more,
-			// in case another builder was added since we last checked
+			// Synchronized builderInitializationLock blocks below are used to avoid
+			// locking during initializeBuilder() call and to make sure two threads
+			// trying to init the same builder in parallel will get properly
+			// initialized builder without deadlocks or ConcurrentModificationException
+			// See bug 538102 and bug 517411.
+			BuilderPersistentInfo info;
+			synchronized (builderInitializationLock) {
+				// get the map of builders to get the last built tree
+				BuilderPersistentInfo builderInitInProgress = getBuilderInitInfo(project, builderName);
+				if (builderInitInProgress != null) {
+					info = builderInitInProgress;
+				} else {
+					info = removePersistentBuilderInfo(builderName, buildConfiguration, buildSpecIndex);
+					setBuilderInitInfo(project, builderName, info);
+				}
+			}
+
 			result = buildCommand.getBuilder(buildConfiguration);
+			if (result == null) {
+				// Not synchronized on builderInitializationLock to avoid deadlocks if the builder init code
+				// requests a lock held by another thread which may be waiting on builderInitializationLock
+				result = initializeBuilder(command, builderName, buildConfiguration, info, status);
+			}
+
+			synchronized (builderInitializationLock) {
+				// the build command holds only one builder per configuration
+				// so query the builder for the configuration once more,
+				// in case another builder was added since we last checked
+				InternalBuilder other = buildCommand.getBuilder(buildConfiguration);
+				if (other == null) {
+					buildCommand.addBuilder(buildConfiguration, (IncrementalProjectBuilder) result);
+				} else {
+					result = other;
+				}
+				setBuilderInitInfo(project, builderName, null);
+			}
 		}
-		// in case the builder was removed, between adding and querying it (see above)
-		if (result == null) {
-			return null;
-		}
+
 		// Ensure the build configuration stays fresh for non-config aware builders
 		result.setBuildConfig(buildConfiguration);
-		if (!validateNature(result, command.getBuilderName())) {
+		if (!validateNature(result, builderName)) {
 			//skip this builder and null its last built tree because it is invalid
 			//if the nature gets added or re-enabled a full build will be triggered
 			result.setLastBuiltTree(null);
@@ -736,23 +771,23 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 				//just return an empty delta rooted at this project
 				return ResourceDeltaFactory.newEmptyDelta(project);
 			}
-			//now check against the cache
-			IResourceDelta result = (IResourceDelta) deltaCache.getDelta(project.getFullPath(), currentLastBuiltTree, currentTree);
-			if (result != null)
-				return result;
 
-			long startTime = 0L;
-			if (Policy.DEBUG_BUILD_DELTA) {
-				startTime = System.currentTimeMillis();
-				Policy.debug("Computing delta for project: " + project.getName()); //$NON-NLS-1$
-			}
-			result = ResourceDeltaFactory.computeDelta(workspace, currentLastBuiltTree, currentTree, project.getFullPath(), -1);
-			deltaCache.cache(project.getFullPath(), currentLastBuiltTree, currentTree, result);
-			if (Policy.DEBUG_BUILD_FAILURE && result == null)
-				Policy.debug("Build: no delta " + debugBuilder() + " [" + debugProject() + "] " + project.getFullPath()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
-			if (Policy.DEBUG_BUILD_DELTA)
-				Policy.debug("Finished computing delta, time: " + (System.currentTimeMillis() - startTime) + "ms" + ((ResourceDelta) result).toDeepDebugString()); //$NON-NLS-1$ //$NON-NLS-2$
-			return result;
+			//now check against the cache
+			IResourceDelta resultDelta = deltaCache.computeIfAbsent(project.getFullPath(), currentLastBuiltTree, currentTree, () -> {
+				long startTime = 0L;
+				if (Policy.DEBUG_BUILD_DELTA) {
+					startTime = System.currentTimeMillis();
+					Policy.debug("Computing delta for project: " + project.getName()); //$NON-NLS-1$
+				}
+				IResourceDelta result = ResourceDeltaFactory.computeDelta(workspace, currentLastBuiltTree, currentTree, project.getFullPath(), -1);
+				if (Policy.DEBUG_BUILD_FAILURE && result == null)
+					Policy.debug("Build: no delta " + debugBuilder() + " [" + debugProject() + "] " + project.getFullPath()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+				if (Policy.DEBUG_BUILD_DELTA)
+					Policy.debug("Finished computing delta, time: " + (System.currentTimeMillis() - startTime) + "ms" + ((ResourceDelta) result).toDeepDebugString()); //$NON-NLS-1$ //$NON-NLS-2$
+
+				return result;
+			});
+			return resultDelta;
 		} finally {
 			lock.release();
 		}
@@ -909,9 +944,9 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 	 * prevent trying to instantiate it every time a build is run.
 	 * This method NEVER returns null.
 	 */
-	private IncrementalProjectBuilder initializeBuilder(String builderName, IBuildConfiguration buildConfiguration, int buildSpecIndex, MultiStatus status) throws CoreException {
+	private InternalBuilder initializeBuilder(ICommand command, String builderName, IBuildConfiguration buildConfiguration, BuilderPersistentInfo info, MultiStatus status) {
 		IProject project = buildConfiguration.getProject();
-		IncrementalProjectBuilder builder = null;
+		InternalBuilder builder = null;
 		try {
 			builder = instantiateBuilder(builderName);
 		} catch (CoreException e) {
@@ -922,22 +957,36 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 			//unable to create the builder, so create a placeholder to fill in for it
 			builder = new MissingBuilder(builderName);
 		}
-		// get the map of builders to get the last built tree
+
+		if (info != null) {
+			ElementTree tree = info.getLastBuiltTree();
+			if (tree != null) {
+				builder.setLastBuiltTree(tree);
+			}
+			builder.setInterestingProjects(info.getInterestingProjects());
+		}
+		builder.setCommand(command);
+		builder.setBuildConfig(buildConfiguration);
+		builder.startupOnInitialize();
+		return builder;
+	}
+
+	private BuilderPersistentInfo removePersistentBuilderInfo(String builderName, IBuildConfiguration buildConfiguration, int buildSpecIndex) throws CoreException {
+		IProject project = buildConfiguration.getProject();
 		ArrayList<BuilderPersistentInfo> infos = getBuildersPersistentInfo(project);
 		if (infos != null) {
 			BuilderPersistentInfo info = getBuilderInfo(infos, builderName, buildConfiguration.getName(), buildSpecIndex);
 			if (info != null) {
 				infos.remove(info);
-				ElementTree tree = info.getLastBuiltTree();
-				if (tree != null)
-					((InternalBuilder) builder).setLastBuiltTree(tree);
-				((InternalBuilder) builder).setInterestingProjects(info.getInterestingProjects());
+				// delete the build map if it's now empty
+				if (infos.isEmpty()) {
+					setBuildersPersistentInfo(project, null);
+				}
+				return info;
 			}
-			// delete the build map if it's now empty
-			if (infos.size() == 0)
-				setBuildersPersistentInfo(project, null);
+
 		}
-		return builder;
+		return null;
 	}
 
 	/**
@@ -1041,17 +1090,18 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 		ElementTree oldTree = builder.getLastBuiltTree();
 		ElementTree newTree = workspace.getElementTree();
 		long start = System.currentTimeMillis();
-		currentDelta = (DeltaDataTree) deltaTreeCache.getDelta(null, oldTree, newTree);
-		if (currentDelta == null) {
+		currentDelta = deltaTreeCache.computeIfAbsent(null, oldTree, newTree, () -> {
 			if (Policy.DEBUG_BUILD_NEEDED) {
-				String message = "Checking if need to build. Starting delta computation between: " + oldTree.toString() + " and " + newTree.toString(); //$NON-NLS-1$ //$NON-NLS-2$
+				String message = "Checking if need to build. Starting delta computation between: " + oldTree + " and " //$NON-NLS-1$ //$NON-NLS-2$
+						+ newTree;
 				Policy.debug(message);
 			}
-			currentDelta = newTree.getDataTree().forwardDeltaWith(oldTree.getDataTree(), ResourceComparator.getBuildComparator());
+			DeltaDataTree computed = newTree.getDataTree().forwardDeltaWith(oldTree.getDataTree(), ResourceComparator.getBuildComparator());
 			if (Policy.DEBUG_BUILD_NEEDED)
 				Policy.debug("End delta computation. (" + (System.currentTimeMillis() - start) + "ms)."); //$NON-NLS-1$ //$NON-NLS-2$
-			deltaTreeCache.cache(null, oldTree, newTree, currentDelta);
-		}
+
+			return computed;
+		});
 
 		//search for the builder's project
 		if (currentDelta.findNodeAt(builder.getProject().getFullPath()) != null) {
@@ -1122,10 +1172,39 @@ public class BuildManager implements ICoreConstants, IManager, ILifecycleListene
 		} catch (CoreException e) {
 			//project is missing -- build state will be lost
 			//can't throw an exception because this happens on startup
-			Policy.log(new ResourceStatus(IStatus.ERROR, 1, project.getFullPath(), "Project missing in setBuildersPersistentInfo", null)); //$NON-NLS-1$
+			logProjectAccessError(project, e, "Project missing in setBuildersPersistentInfo"); //$NON-NLS-1$
 		}
 	}
 
+	private void setBuilderInitInfo(IProject project, String builderName, BuilderPersistentInfo info) {
+		try {
+			project.setSessionProperty(keyForBuilderInfo(builderName), info);
+		} catch (CoreException e) {
+			//project is missing -- build state will be lost
+			//can't throw an exception because this happens on startup
+			logProjectAccessError(project, e, "Project missing in setBuilderInitInfo"); //$NON-NLS-1$
+		}
+	}
+
+	private BuilderPersistentInfo getBuilderInitInfo(IProject project, String builderName) {
+		try {
+			return (BuilderPersistentInfo) project.getSessionProperty(keyForBuilderInfo(builderName));
+		} catch (CoreException e) {
+			//project is missing -- build state will be lost
+			//can't throw an exception because this happens on startup
+			logProjectAccessError(project, e, "Project missing in getBuilderInitInfo"); //$NON-NLS-1$
+		}
+		return null;
+	}
+
+	private void logProjectAccessError(IProject project, CoreException e, String message) {
+		Policy.log(new ResourceStatus(IStatus.ERROR, 1, project.getFullPath(), message, e));
+	}
+
+	private static QualifiedName keyForBuilderInfo(String builderName) {
+		return new QualifiedName(ResourcesPlugin.PI_RESOURCES, BUILDER_INIT + builderName);
+	}
+
 	@Override
 	public void shutdown(IProgressMonitor monitor) {
 		autoBuildJob.cancel();
