@@ -152,8 +152,7 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 	private IProject[] copyAndRemoveDuplicates(IProject[] projects) {
 		IProject[] result = new IProject[projects.length];
 		int count = 0;
-		next: for (int i = 0; i < projects.length; i++) {
-			IProject project = projects[i];
+		next: for (IProject project : projects) {
 			// scan to see if there are any other projects by the same name
 			for (int j = 0; j < count; j++)
 				if (project.equals(result[j]))
@@ -205,9 +204,9 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 	 * Returns the union of the description's static and dynamic project references,
 	 * with duplicates omitted. The calculation is optimized by caching the result
 	 * Call the configuration based implementation.
-	 * @see #getAllBuildConfigReferences(String, boolean)
+	 * @see #getAllBuildConfigReferences(IProject, String, boolean)
 	 */
-	public IProject[] getAllReferences(boolean makeCopy) {
+	public IProject[] getAllReferences(IProject project, boolean makeCopy) {
 		int dirtyCount;
 		IProject[] projRefs;
 
@@ -221,12 +220,12 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 		while (projRefs == null) {
 			IBuildConfiguration[] refs;
 			if (hasBuildConfig(activeConfiguration))
-				refs = getAllBuildConfigReferences(activeConfiguration, false);
+				refs = getAllBuildConfigReferences(project, activeConfiguration, false);
 			else if (configNames.length > 0)
-				refs = getAllBuildConfigReferences(configNames[0], false);
+				refs = getAllBuildConfigReferences(project, configNames[0], false);
 			else
 				// No build configuration => fall-back to default
-				refs = getAllBuildConfigReferences(IBuildConfiguration.DEFAULT_CONFIG_NAME, false);
+				refs = getAllBuildConfigReferences(project, IBuildConfiguration.DEFAULT_CONFIG_NAME, false);
 			Collection<IProject> l = getProjectsFromBuildConfigRefs(refs);
 
 			synchronized (cachedRefsMutex) {
@@ -254,14 +253,13 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 	 * be resolved using {@link BuildConfiguration#getBuildConfig()} before use.
 	 * Returns an empty array if the given configName does not exist in the description.
 	 */
-	public IBuildConfiguration[] getAllBuildConfigReferences(String configName, boolean makeCopy) {
+	public IBuildConfiguration[] getAllBuildConfigReferences(IProject project, String configName, boolean makeCopy) {
 		if (!hasBuildConfig(configName))
 			return EMPTY_BUILD_CONFIG_REFERENCE_ARRAY;
 		IBuildConfiguration[] refs = cachedConfigRefs.get(configName);
 		if (refs == null) {
 			Set<IBuildConfiguration> references = new LinkedHashSet<>();
 			IBuildConfiguration[] dynamicBuildConfigs = dynamicConfigRefs.containsKey(configName) ? dynamicConfigRefs.get(configName) : EMPTY_BUILD_CONFIG_REFERENCE_ARRAY;
-			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(getName());
 			Collection<BuildConfiguration> dynamic;
 			try {
 				IBuildConfiguration buildConfig = project.getBuildConfig(configName);
@@ -725,7 +723,7 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 				//not actually changed anything
 				return false;
 			}
-			linkDescriptions = newMap.size() == 0 ? null : newMap;
+			linkDescriptions = newMap.isEmpty() ? null : newMap;
 		}
 		return true;
 	}
@@ -755,9 +753,9 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 			LinkedList<FilterDescription> descList = filterDescriptions.get(path);
 			if (descList != null) {
 				descList.remove(description);
-				if (descList.size() == 0) {
+				if (descList.isEmpty()) {
 					filterDescriptions.remove(path);
-					if (filterDescriptions.size() == 0)
+					if (filterDescriptions.isEmpty())
 						filterDescriptions = null;
 				}
 			}
@@ -798,7 +796,7 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 				//not actually changed anything
 				return false;
 			}
-			variableDescriptions = newMap.size() == 0 ? null : newMap;
+			variableDescriptions = newMap.isEmpty() ? null : newMap;
 		}
 		return true;
 	}
@@ -829,7 +827,7 @@ public class ProjectDescription extends ModelObject implements IProjectDescripti
 				//not actually changed anything
 				return false;
 			}
-			if (filterDescriptions.size() == 0)
+			if (filterDescriptions.isEmpty())
 				filterDescriptions = null;
 		}
 		return true;
