@@ -23,6 +23,9 @@ package org.eclipse.core.internal.resources;
 import java.io.*;
 import java.io.File;
 import java.net.URI;
+import java.nio.charset.Charset;
+import java.nio.file.Files;
+import java.nio.file.Paths;
 import java.util.*;
 import java.util.zip.*;
 import org.eclipse.core.filesystem.EFS;
@@ -59,7 +62,7 @@ public class SaveManager implements IElementInfoFlattener, IManager, IStringPool
 		}
 	}
 
-	protected static final String ROOT_SEQUENCE_NUMBER_KEY = Path.ROOT.toString() + LocalMetaArea.F_TREE;
+	protected static final String ROOT_SEQUENCE_NUMBER_KEY = Path.ROOT + LocalMetaArea.F_TREE;
 	protected static final String CLEAR_DELTA_PREFIX = "clearDelta_"; //$NON-NLS-1$
 	protected static final String DELTA_EXPIRATION_PREFIX = "deltaExpiration_"; //$NON-NLS-1$
 	protected static final int DONE_SAVING = 3;
@@ -840,12 +843,8 @@ public class SaveManager implements IElementInfoFlattener, IManager, IStringPool
 			if (!target.exists())
 				return;
 		}
-		try {
-			try (
-				SafeChunkyInputStream input = new SafeChunkyInputStream(target);
-			) {
-				masterTable.load(input);
-			}
+		try (SafeChunkyInputStream input = new SafeChunkyInputStream(target)) {
+			masterTable.load(input);
 		} catch (IOException e) {
 			String message = Messages.resources_exMasterTable;
 			throw new ResourceException(IResourceStatus.INTERNAL_ERROR, null, message, e);
@@ -1035,12 +1034,8 @@ public class SaveManager implements IElementInfoFlattener, IManager, IStringPool
 			savedStates = Collections.synchronizedMap(new HashMap<String, SavedState>(10));
 			return;
 		}
-		try {
-			try (
-				DataInputStream input = new DataInputStream(new SafeFileInputStream(treeLocation.toOSString(), tempLocation.toOSString(), TREE_BUFFER_SIZE));
-			) {
-				WorkspaceTreeReader.getReader(workspace, input.readInt()).readTree(input, monitor);
-			}
+		try (DataInputStream input = new DataInputStream(new SafeFileInputStream(treeLocation.toOSString(), tempLocation.toOSString(), TREE_BUFFER_SIZE))) {
+			WorkspaceTreeReader.getReader(workspace, input.readInt()).readTree(input, monitor);
 		} catch (IOException e) {
 			String msg = NLS.bind(Messages.resources_readMeta, treeLocation.toOSString());
 			throw new ResourceException(IResourceStatus.FAILED_READ_METADATA, treeLocation, msg, e);
@@ -1546,7 +1541,8 @@ public class SaveManager implements IElementInfoFlattener, IManager, IStringPool
 					parent = parent.getParent();
 				}
 				if (parent == null) {
-					IStatus status = new Status(IStatus.WARNING, ResourcesPlugin.PI_RESOURCES, IResourceStatus.INTERNAL_ERROR, "null parent found while collapsing trees", null); //$NON-NLS-1$
+					Exception e = new NullPointerException("null parent found while collapsing trees"); //$NON-NLS-1$
+					IStatus status = new Status(IStatus.WARNING, ResourcesPlugin.PI_RESOURCES, IResourceStatus.INTERNAL_ERROR, e.getMessage(), e);
 					Policy.log(status);
 					return null;
 				}
@@ -1592,13 +1588,46 @@ public class SaveManager implements IElementInfoFlattener, IManager, IStringPool
 					int valueInMemory = Integer.parseInt(masterTable.getProperty(ROOT_SEQUENCE_NUMBER_KEY));
 					// new master table must provide greater or equal sequence number for root
 					// throw exception if new value is lower than previous one - we cannot allow to desynchronize master table on disk
-					String message = "Cannot set lower sequence number for root (previous: " + valueInFile + ", new: " + valueInMemory + "). Location: " + target.getAbsolutePath(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
-					Assert.isLegal(valueInMemory >= valueInFile, message);
+					if (valueInMemory < valueInFile) {
+						String message = getBadSequenceNumberErrorMessage(target, valueInFile, valueInMemory, masterTable, previousMasterTable);
+						Assert.isLegal(false, message);
+					}
 				}
 			}
 		}
 	}
 
+	private static String getBadSequenceNumberErrorMessage(java.io.File target, int valueInFile, int valueInMemory, MasterTable currentMasterTable, MasterTable previousMasterTable) {
+		StringBuilder messageBuffer = new StringBuilder();
+		messageBuffer.append("Cannot set lower sequence number for root (previous: "); //$NON-NLS-1$
+		messageBuffer.append(valueInFile);
+		messageBuffer.append(", new: "); //$NON-NLS-1$
+		messageBuffer.append(valueInMemory);
+		messageBuffer.append("). Location: "); //$NON-NLS-1$
+		messageBuffer.append(target.getAbsolutePath());
+		try {
+			messageBuffer.append("Timestamps and tree sequence numbers from file:"); //$NON-NLS-1$
+			java.nio.file.Path targetPath = Paths.get(target.getAbsolutePath());
+			List<String> masterTableFileContents = Files.readAllLines(targetPath, Charset.defaultCharset());
+			for (String line : masterTableFileContents) {
+				if (line != null) {
+					boolean isPropertiesTimestamp = line.startsWith("#"); //$NON-NLS-1$
+					boolean isTreeProperty = line.startsWith(ROOT_SEQUENCE_NUMBER_KEY);
+					if (isPropertiesTimestamp || isTreeProperty) {
+						messageBuffer.append(System.lineSeparator());
+						messageBuffer.append(line);
+					}
+				}
+			}
+		} catch (IOException e) {
+			ILog log = ResourcesPlugin.getPlugin().getLog();
+			String errorMessage = "Error occurred while reading master table file"; //$NON-NLS-1$
+			IStatus errorStatus = new Status(IStatus.ERROR, ResourcesPlugin.PI_RESOURCES, errorMessage, e);
+			log.log(errorStatus);
+		}
+		return messageBuffer.toString();
+	}
+
 	/**
 	 * Visit the given resource (to depth infinite) and write out extra information
 	 * like markers and sync info. To be called during a full save and project save.
