@@ -842,9 +842,9 @@ public class ProjectPreferencesTest extends ResourceTest {
 		try {
 			File folder = new File(project.getLocation().toOSString() + "/.settings");
 			folder.mkdir();
-			BufferedWriter bw = new BufferedWriter(new FileWriter(folder.getPath() + "/" + nodeName + ".prefs"));
-			bw.write("#Fri Jan 28 10:28:45 CET 2011\neclipse.preferences.version=1\nKEY=VALUE");
-			bw.close();
+			try (BufferedWriter bw = new BufferedWriter(new FileWriter(folder.getPath() + "/" + nodeName + ".prefs"))) {
+				bw.write("#Fri Jan 28 10:28:45 CET 2011\neclipse.preferences.version=1\nKEY=VALUE");
+			}
 		} catch (IOException e) {
 			fail("2.0", e);
 		}
@@ -880,16 +880,17 @@ public class ProjectPreferencesTest extends ResourceTest {
 		//preferences were changed so the new file should contain two lines: 'KEY=VALUE' and 'NEW_KEY=NEW_VALUE'
 		try {
 			File folder = new File(project.getLocation().toOSString() + "/.settings");
-			BufferedReader br = new BufferedReader(new FileReader(folder.getPath() + "/" + nodeName + ".prefs"));
-			List<String> lines = new ArrayList<>();
-			String line = br.readLine();
-			while (line != null) {
-				if ((!line.startsWith("#")) && (!line.startsWith("eclipse.preferences.version"))) {
-					lines.add(line);
+			List<String> lines;
+			try (BufferedReader br = new BufferedReader(new FileReader(folder.getPath() + "/" + nodeName + ".prefs"))) {
+				lines = new ArrayList<>();
+				String line = br.readLine();
+				while (line != null) {
+					if ((!line.startsWith("#")) && (!line.startsWith("eclipse.preferences.version"))) {
+						lines.add(line);
+					}
+					line = br.readLine();
 				}
-				line = br.readLine();
 			}
-			br.close();
 			assertEquals(2, lines.size());
 			Collections.sort(lines);
 			assertTrue(lines.get(0).equals("KEY=VALUE"));
@@ -1006,9 +1007,9 @@ public class ProjectPreferencesTest extends ResourceTest {
 		//create file with preferences that will be discovered during refresh
 		File folder = new File(project.getLocation().toOSString() + "/.settings");
 		folder.mkdir();
-		BufferedWriter bw = new BufferedWriter(new FileWriter(folder.getPath() + "/" + nodeName + ".prefs"));
-		bw.write("#Fri Jan 28 10:28:45 CET 2011\neclipse.preferences.version=1\nKEY=VALUE");
-		bw.close();
+		try (BufferedWriter bw = new BufferedWriter(new FileWriter(folder.getPath() + "/" + nodeName + ".prefs"))) {
+			bw.write("#Fri Jan 28 10:28:45 CET 2011\neclipse.preferences.version=1\nKEY=VALUE");
+		}
 
 		//create project preference node
 		Preferences projectNode = Platform.getPreferencesService().getRootNode().node(ProjectScope.SCOPE).node(projectName);
